body,
html {
    margin: 0;
    padding: 0;
}

#stpyvistadiv {
    text-align: center;
    display: block;
    margin-left: 0px;
    margin-right: 0px;
    width: 100%;
    background-color: rgba(255, 0, 0, 0);
}